# -*- coding: utf-8 -*-
# Generated by Django 1.9.7.dev20160506141209 on 2016-05-10 10:41
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('raster', '0032_auto_20160426_0152'),
    ]

    operations = [
        migrations.AddField(
            model_name='rasterlayer',
            name='build_pyramid',
            field=models.BooleanField(default=True, help_text='Should the tile pyramid be built? If unchecked, tiles will only be generated at the max zoom level.'),
        ),
        migrations.AddField(
            model_name='rasterlayer',
            name='next_higher',
            field=models.BooleanField(default=True, help_text='Compared to the scale of the rasterlayer, use the next-higher zoomlevel as max zoom? If unchecked, the next-lower zoom level is used. This flag is ignored if the max_zoom is manually specified.'),
        ),
        migrations.AddField(
            model_name='rasterlayer',
            name='source_url',
            field=models.URLField(blank=True, default='', help_text='External url to get the raster file from. If a value is set,the rasterfile field will be ignored.', max_length=2500),
        ),
        migrations.AddField(
            model_name='rasterlayer',
            name='store_reprojected',
            field=models.BooleanField(default=True, help_text='Should the reprojected raster be stored? If unchecked, the reprojected version of the raster is not stored.'),
        ),
    ]
