# -*- coding: utf-8 -*-
# Generated by Django 1.9b1 on 2015-11-10 07:56
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('raster', '0020_auto_20150928_0408'),
    ]

    operations = [
        migrations.CreateModel(
            name='RasterLayerParseStatus',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.IntegerField(choices=[(0, b'Layer not yet parsed'), (1, b'Downloading file'), (2, b'Reprojecting'), (3, b'Creating tiles'), (4, b'Dropping empty tiles'), (5, b'Finished parsing'), (6, b'Failed parsing')], default=0)),
                ('tile_level', models.IntegerField(blank=True, null=True)),
                ('log', models.TextField(default=b'', editable=False)),
                ('rasterlayer', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='parsestatus', to='raster.RasterLayer')),
            ],
        ),
    ]
