# -*- coding: utf-8 -*-
# Generated by Django 1.11.2.dev20170508150049 on 2017-05-09 05:59
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('raster', '0036_auto_20170509_0548'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='legendentryorder',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='legendentryorder',
            name='legend',
        ),
        migrations.RemoveField(
            model_name='legendentryorder',
            name='legendentry',
        ),
        migrations.RemoveField(
            model_name='legend',
            name='entries',
        ),
        migrations.DeleteModel(
            name='LegendEntryOrder',
        ),
    ]
